@@ -6,7 +6,7 @@ Mojolicious::Plugin::LinkEmbedder - Convert a URL to embedded content
 
 =head1 VERSION
 
-0.21
+0.22
 
 =head1 DESCRIPTION
 
@@ -20,10 +20,6 @@ content.
   use Mojolicious::Lite;
   plugin LinkEmbedder => { route => '/embed' };
 
-The simple route includes the caching example below.
-
-Caching is EXPERIMENTAL and could be removed without notice.
-
 =head2 Full control
 
   plugin 'LinkEmbedder';
@@ -59,20 +55,19 @@ Caching is EXPERIMENTAL and could be removed without notice.
   get '/embed' => sub {
     my $c = shift;
     my $url = $c->param('url');
-    my $cache = $c->app->{linkembedder_cache} ||= Mojo::Cache->new;
     my $cached;
 
     $c->delay(
       sub {
         my ($delay) = @_;
-        return $delay->pass($cached) if $cached = $cache->get($url);
+        return $delay->pass($cached) if $cached = $c->cache->get($url);
         return $c->embed_link($c->param('url'), $delay->begin);
       },
       sub {
         my ($delay, $link) = @_;
 
         $link = $link->TO_JSON if UNIVERSAL::can($link, 'TO_JSON');
-        $cache->set($url => $link);
+        $c->cache->set($url => $link);
 
         $c->respond_to(
           json => {
@@ -135,6 +130,8 @@ Caching is EXPERIMENTAL and could be removed without notice.
 
 =item * L<Mojolicious::Plugin::LinkEmbedder::Link::Text::Pastebin>
 
+=item * L<Mojolicious::Plugin::LinkEmbedder::Link::Text::Pastie>
+
 =item * L<Mojolicious::Plugin::LinkEmbedder::Link::Text::PasteScsysCoUk>
 
 =item * L<Mojolicious::Plugin::LinkEmbedder::Link::Text::Twitter>
@@ -146,13 +143,12 @@ Caching is EXPERIMENTAL and could be removed without notice.
 =cut
 
 use Mojo::Base 'Mojolicious::Plugin';
-use Mojo::Cache;
 use Mojo::JSON;
 use Mojo::UserAgent;
 use Mojolicious::Plugin::LinkEmbedder::Link;
 use constant DEBUG => $ENV{MOJO_LINKEMBEDDER_DEBUG} || 0;
 
-our $VERSION = '0.21';
+our $VERSION = '0.22';
 
 has _ua => sub { Mojo::UserAgent->new(max_redirects => 3) };
 
@@ -167,7 +163,7 @@ See L</SYNOPSIS>.
 sub embed_link {
   my ($self, $c, $url, $cb) = @_;
 
-  $url = Mojo::URL->new($url) unless ref $url;
+  $url = Mojo::URL->new($url || '') unless ref $url;
 
   if ($url =~ m!\.(?:jpg|png|gif)\b!i) {
     return $c if $self->_new_link_object(image => $c, {url => $url}, $cb);
@@ -175,9 +171,18 @@ sub embed_link {
   if ($url =~ m!\.(?:mpg|mpeg|mov|mp4|ogv)\b!i) {
     return $c if $self->_new_link_object(video => $c, {url => $url}, $cb);
   }
-  if ($url =~ m!^spotify:!i) {
+  if ($url =~ m!^spotify:\w+!i) {
     return $c if $self->_new_link_object('open.spotify' => $c, {url => $url}, $cb);
   }
+  if (!$url or !$url->host) {
+    return $c->tap(
+      $cb,
+      Mojolicious::Plugin::LinkEmbedder::Link->new(
+        url   => Mojo::URL->new,
+        error => {message => 'Invalid input', code => 400,}
+      )
+    );
+  }
 
   return $c->delay(
     sub {
@@ -193,9 +198,16 @@ sub embed_link {
 
 sub _learn {
   my ($self, $c, $tx, $cb) = @_;
-  my $ct = $tx->res->headers->content_type || '';
-  my $url = $tx->req->url;
+  my $ct   = $tx->res->headers->content_type || '';
+  my $etag = $tx->res->headers->etag;
+  my $url  = $tx->req->url;
 
+  if ($etag and $etag eq ($c->req->headers->etag // '')) {
+    return $c->$cb(Mojolicious::Plugin::LinkEmbedder::Link->new(_tx => $tx, etag => $etag));
+  }
+  if (my $err = $tx->error) {
+    return $c->$cb(Mojolicious::Plugin::LinkEmbedder::Link->new(_tx => $tx, error => $err));
+  }
   if (my $type = lc $url->host) {
     $type =~ s/^(?:www|my)\.//;
     $type =~ s/\.\w+$//;
@@ -278,6 +290,7 @@ sub register {
     'open.spotify'   => 'Mojolicious::Plugin::LinkEmbedder::Link::Music::Spotify',
     'paste.scsys.co' => 'Mojolicious::Plugin::LinkEmbedder::Link::Text::PasteScsysCoUk',
     'pastebin'       => 'Mojolicious::Plugin::LinkEmbedder::Link::Text::Pastebin',
+    'pastie'         => 'Mojolicious::Plugin::LinkEmbedder::Link::Text::Pastie',
     'ted'            => 'Mojolicious::Plugin::LinkEmbedder::Link::Video::Ted',
     'text'           => 'Mojolicious::Plugin::LinkEmbedder::Link::Text',
     'twitter'        => 'Mojolicious::Plugin::LinkEmbedder::Link::Text::Twitter',
@@ -296,35 +309,44 @@ sub register {
   );
 
   if (my $route = $config->{route}) {
-    $self->_add_action($app, $route);
+    $self->_add_action($app, $route, $config);
   }
 }
 
 sub _add_action {
-  my ($self, $app, $route) = @_;
+  my ($self, $app, $route, $config) = @_;
 
-  unless (ref $route) {
-    $route = $app->routes->route($route);
-  }
+  $config->{max_age} //= 60;
+  $route = $app->routes->route($route) unless ref $route;
 
   $route->to(
     cb => sub {
-      my $c     = shift;
-      my $url   = $c->param('url');
-      my $cache = $c->app->{'linkembedder.cache'} ||= Mojo::Cache->new;
-      my $cached;
+      my $c             = shift;
+      my $url           = $c->param('url');
+      my $if_none_match = $c->req->headers->if_none_match;
 
       $c->delay(
         sub {
           my ($delay) = @_;
-          return $delay->pass($cached) if $cached = $cache->get($url);
-          return $c->embed_link($c->param('url'), $delay->begin);
+          $c->embed_link($url, $delay->begin);
         },
         sub {
           my ($delay, $link) = @_;
-          $link = $link->TO_JSON if UNIVERSAL::can($link, 'TO_JSON');
-          $cache->set($url => $link);
-          $c->respond_to(json => {json => $link}, any => {text => $link->{html}});
+          my $err = $link->error;
+
+          if ($err) {
+            $c->res->code($err->{code} || 500);
+            $c->respond_to(json => {json => $err}, any => {text => $err->{message} || 'Unknown error.'});
+          }
+          elsif ($if_none_match and $if_none_match eq $link->etag) {
+            $c->res->code(304);
+            $c->rendered;
+          }
+          else {
+            $c->res->headers->etag($link->etag) if $link->etag;
+            $c->res->headers->cache_control("max-age=$config->{max_age}") if !$link->etag and $config->{max_age};
+            $c->respond_to(json => {json => $link}, any => {text => $link->to_embed});
+          }
         }
       );
     }
