@@ -45,19 +45,29 @@ sub provider_name {'scsys.co.uk'}
 
 sub learn {
   my ($self, $c, $cb) = @_;
-  my $media_id = $self->media_id or return $self->SUPER::learn($c, $cb);
-  my $url = Mojo::URL->new('http://paste.scsys.co.uk');
+  my $raw_url = $self->raw_url or return $self->SUPER::learn($c, $cb);
 
   $self->ua->get(
-    $url->path($media_id)->query(tx => 'on'),
+    $raw_url,
     sub {
       my ($ua, $tx) = @_;
-      $self->{text} = $tx->res->body if $tx->success;
+      $self->{text} = Mojo::Util::xml_escape($tx->res->body) if $tx->success;
       $self->$cb;
     },
   );
 }
 
+=head2 raw_url
+
+=cut
+
+sub raw_url {
+  my $self = shift;
+  my $media_id = $self->media_id or return;
+
+  Mojo::URL->new("http://paste.scsys.co.uk/$media_id?tx=on");
+}
+
 =head1 AUTHOR
 
 Jan Henning Thorsen
